import Component from "../../../../src/component";

import { isObject, isString, isFunction, isEqualTo, isUndefined, isBoolean, noop } from '../../../../src/assertions';
import IDOM from "incremental-dom";

const { patch, text, elementOpen, elementClose, elementVoid, applyProp, attributes } = IDOM;
IDOM.attributes.value = function (el, name, value) {
  console.log('value')
  el.value = value
}
const ToDoItemTemplate = data => htmlToNode(
  `<item><b>${data.id}</b> ${data.value}</item>`
);


const _parseOptions = options => {

  isObject(options, 'options');

  return options;
};

const TodoComponent = (options = {}) =>  {

  const { observer } = _parseOptions(options);

  return Component({
    observer,
    tagName: 'todo-list',
    initialState: {
      todos: [
        {id: 1, value: 'ga'},
        {id: 2, value: 'bu'},
        {id: 3, value: 'zo'},
        {id: 4, value: 'meu'}
      ]
    },
    connect(element) {

      console.error('TodoList.connect()');

      let todoId = 0;


      const btn = element.querySelector('button');
      const input = element.querySelector('input');

      btn.addEventListener('click', () => {

        const { value } = input;

        if (value.length > 0) {

          const state = element.state;

          state.todos.push({
            id: ++todoId,
            value
          });

          element.state = state;
        }
      });

    },
    removeTodo(element, id) {

      const state = element.state;
      const index = state.todos.findIndex(todo => todo.id === id);

      console.log('removeTodo', index, state.todos);

      state.todos.splice(index, 1);

      element.state = state;
    },
    render(element, { todos }, { removeTodo }) {

      console.error('Todo.render()');

      patch(element, () => {
        elementOpen('ul');
        todos.forEach(todo => {
          elementOpen('li', todo.id, null, 'onclick', () => removeTodo(todo.id));
          elementOpen('strong');
          text(todo.id);
          elementClose('strong');
          text(todo.value);

          elementClose('li')
        });
        elementClose('ul');

        elementVoid('input', null, null, 'value', new String());
        elementOpen('button');
        text('Add todo');
        elementClose('button');
      });

    }
  });
};

export default TodoComponent;