'use strict';

const internals = {};

internals.Checker = (check, errorMessage) => {

  return (input, assert) => {

    const isTrue = !!check(input);
    if (isTrue)
      return input || true;

    if (!assert)
      return false;

    throw new Error(`"${typeof assert !== 'string' ? 'input' : assert}" ${errorMessage}`);
  };
};
export const noop = () => {};
export const returnTrue = () => true;
export const isEqualTo = value => internals.Checker(input => input === value, 'is not equal to value');
export const isTrue = isEqualTo(true);
export const isFalse = isEqualTo(false);
export const isFalsy = internals.Checker(input => !input, 'is not a falsy value');
export const isRequired = internals.Checker(input => !!input, 'is required');
export const isInstanceOf = type => internals.Checker(input => input instanceof type, `is not an instance of ${type.name}`);
export const isObject = internals.Checker(input => typeof input === 'object', 'must be an object');
export const isString = internals.Checker(input => typeof input === 'string', 'must be a string');
export const isFunction = internals.Checker(input => typeof input === 'function', 'must be a string');

export const before = (object, key, whenCalled) => {

  isObject(object, 'object');
  isRequired(key, 'key');
  isString(key, 'key');
  isFunction(whenCalled, 'whenCalled');

  const oldFunction = object[key];

  isFunction(oldFunction, key || true);


  object[key] = (...args) => {
    whenCalled(...args);
    return oldFunction(...args);
  };
};