import Emitter from 'klak';
import { isObject, isString, isFunction, isElement, isUndefined, isBoolean, noop, identity } from './assertions';

const Observer = (element, options) => {

  if(document.readyState !== 'interactive' && document.readyState !== 'complete')
    throw new Error(`The DOM must be loaded and parsed`);

  const _observer = new MutationObserver(mutations => {


    console.log('mutations', mutations)
    mutations.forEach(mutation => {

      const {
        type,
        target,
        attributeName:name,
        attributeNamespace:namespace,
        addedNodes,
        removedNodes,
        oldValue
      } = mutation;


      if (type === 'attributes') {

        // observer.attributeHasChanged({
        //   element: target,
        //   name,
        //   namespace,
        //   oldValue
        // });

        observer.attributesHasChanged.filter(callback => {

          return callback.tagName === target.tagName.toLowerCase();
        }).forEach(callback => {

          callback({
            element: target,
            name,
            namespace,
            oldValue
          });
        });


        return;
      }

      if (type === 'childList') {

        if (addedNodes.length > 0) {
          Array.from(addedNodes).filter(isElement).forEach(element => {
            observer.wasAttached.filter(callback => {

              return callback.tagName === element.tagName.toLowerCase();
            }).forEach(callback => {

              console.log('attached', element.tagName);

              callback(element);
            });
          })
        }

        if (removedNodes.length > 0) {
          Array.from(removedNodes).filter(isElement).forEach(element => {
            observer.wasDetached.filter(callback => {


              return callback.tagName === element.tagName.toLowerCase();
            }).forEach(callback => {

              console.log('detached', element.tagName);

              callback(element);
            });
          })
        }

        return;
      }

      throw new Error(`Not yet implemented :)`);
    });
  });

  const observer = {
    isPwetObserver: true,
    connect: () => _observer.observe(element, options),
    disconnect: _observer.disconnect.bind(_observer),
    takeRecords: _observer.takeRecords.bind(_observer),
    wasAttached: [],
    wasDetached: [],
    attributesHasChanged: []
  };

  return observer;
};

export default Observer;