import {
  assert,
  isElement,
  isUnknownElement,
  isComponent,
  isString,
  isObject,
  isFunction,
  isUndefined,
  isArray
} from '../../../src/assertions';

import { clone, ByFilter, isAttached } from '../../../src/utilities';
import Emitter from 'klak';

const internals = {
  // createElement: document.createElement.bind(document),
  // createElementNS: document.createElementNS.bind(document),
  // setAttribute: HTMLElement.prototype.setAttribute,

  mutationCounter: 0
};
internals.tagNameFilter = ByFilter.bind(null, 'tagName');

// HTMLElement.prototype.setAttribute = function (...args) {
//   log('setAttribute()', isAttached(this), ...args);
//
//   return internals.setAttribute.call(this, ...args);
// };


// document.createElementNS = (namespace, tagName) => {
//   // log('document.createElementNS()', tagName);
//
//   const element = internals.createElementNS(namespace, tagName);
//
//   if (!isUnknownElement(element))
//     return element;
//
//   return internals.Component(element);
// };
// document.createElement = (tagName) => {
//   // log('document.createElement()', tagName);
//   const element = internals.createElement(tagName);
//
//   if (!isUnknownElement(element))
//     return element;
//
//   return internals.Component(element);
// };

internals.Updater = (element, newState) => {

  return true;

};

internals.Definition = options => {

  assert(isObject(options), `'definition' must be an object`);

  const {
    tagName,
    render,
    update = internals.Updater,
    attributes = {},
    ...properties
  } = options;

  assert(isString(tagName), `'tagName' must be a string`);
  assert(isFunction(render), `'render' must be a function`);
  assert(isFunction(update), `'update' must be a function`);

  const methods = {};

  Object.keys(properties).reduce((state, key) => {

    if (!isString(key))
      return;

    let value = properties[key];

    if (isFunction(value)) {

      methods[key] = value;

      return state;
    }

    return Object.assign(state, { [key]: value })
  }, attributes);


  return {
    tagName,
    attributes,
    methods,
    render,
    update
  };
};

internals.Observer = () => {

  if (internals.observer === void 0) {

    internals.observer = new MutationObserver(mutations => {
      const i = ++internals.mutationCounter;

      mutations.forEach(mutation => {

        const { type, target, addedNodes, removedNodes } = mutation;


        const addedElements = Array.from(addedNodes).filter(isElement);
        const removedElements = Array.from(removedNodes).filter(isElement);

        // log('mutation_'+i, type, target, addedElements, removedElements);

        if (type === 'attributes') {

          // observer.attributeHasChanged({
          //   element: target,
          //   name,
          //   namespace,
          //   oldValue
          // });
          log('attributes', mutation);
          // observer.attributesHasChanged.filter(callback => {
          //
          //   return callback.tagName === target.tagName.toLowerCase();
          // }).forEach(callback => {
          //
          //   callback({
          //     element: target,
          //     name,
          //     namespace,
          //     oldValue
          //   });
          // });


          return;
        }

        if (type === 'childList') {

          addedElements.filter(isComponent).forEach(element => {

            const tagName = element.tagName.toLowerCase();
            error(`Observer: Component(${tagName}) was attached`, element.state);

            internals.Component.emit('attached', {
              tagName,
              element
            });

            const state = element.state;
            for (let { name, value } of element.attributes) {

              if (!isUndefined(state[name])) {

                state[name] = value;
                continue;
              }

              // value = state[name];
              // element.removeAttribute(name);
            }
            // error(`Observer: Component(${tagName}) was attached`, state, element.state);

            update(state);

          });
          //
          // removedElements.forEach(element => {
          //   const tagName = element.tagName.toLowerCase();
          //
          //   internals.definitions.filter(definiton => {
          //
          //     return definiton.tagName === tagName
          //   }).forEach(found => element.render(element));
          //
          //
          // })
        }

      })
    });

    internals.observer.observe(document.body, { childList: true, subtree: true });
  }

  return internals.observer;
};

internals.Component = module.exports = (element) => {

  // if (isComponent(element)) {}
  //   return element.render();

  assert(!isComponent(element), `'element' is already a PwetComponent`);

  const tagName = element.tagName.toLowerCase();

  const def = internals.definitions.find(internals.tagNameFilter(tagName));

  if (isUndefined(def))
    return element;


  const { render:_render, update:_update, attributes, methods } = def;

  const state = element.state;

  error(`Component(${tagName})`, attributes, state);

  Object.keys(methods).reduce((methods, key) => {

    return Object.assign(methods, { [key]: methods[key].bind(null, element) })
  }, methods);

  element.render = () => {
    error(`Component(${tagName}).render()`, element.state, isAttached(element));

    element.innerHTML = 'YO'
    // _render(element, element.state, methods);


  };
  element.update = (newState) => {
    error(`Component(${tagName}).update()`, newState.gna, isAttached(element));

    const shouldBeRendered = def.update(element, newState);

    element[internals.$state] = newState;

    if (shouldBeRendered && isAttached(element))
      element.render();

  };

  Object.defineProperty(element, 'state', {
    get() {
      return element[internals.$state];
    },
    set: element.update
  });
  // element.innerHTML = 'WOO !!';

  element.state = Object.keys(def.attributes).reduce((state, key) => {

    const attribute = def.attributes[key];

    // Object.defineProperty(element, key, {
    //   get() {
    //     return element.state[key];
    //   },
    //   set(newValue) {
    //
    //     log('setAttribute()', key, newValue, isAttached(element));
    //
    //     element.state = Object.assign(element.state, {
    //       [key]: newValue
    //     });
    //   }
    // });

    return Object.assign(state, {
      [key]: element.getAttribute(key) || clone(attribute)
    });
  }, {});

  element.isPwetComponent = true;
  return element;
};


Object.assign(internals.Component, Emitter(['attached', 'detached']));

internals.Component.initialize = () => {
  error(`Component.initialize()`);

  if(document.readyState !== 'interactive' && document.readyState !== 'complete')
    throw new Error(`Cannot initialize component because the DOM is not loaded and parsed`);

  // internals.Observer();

  internals.definitions = [];
};
// (tagName, options) => {
//
//   const element = internals.createElement(tagName, options);
//
//   return element;
// };



internals.Component.define = (definition) => {

  if (internals.definitions === void 0) {
    internals.Component.initialize();
  }


  definition = internals.Definition(definition);

  internals.definitions.push(definition);

  error(`Component defined (${definition.tagName})`);


  // customElements.define(definition.tagName, class extends HTMLElement {
  //   constructor() {
  //     const element = super();
  //     error('HTMLElement()', element);
  //     // return internals.Component(element)
  //   }
  //   connectedCallback() {
  //     error('HTMLElement.connected()', this);
  //
  //     this.innerHTML = 'pkpa'
  //     // const state = this.state;
  //     // // if (state) {
  //     //   // delete this.state;
  //     //   this.state = state;
  //     // // }
  //     //
  //
  //   }
  // });

  customElements.define(definition.tagName, class extends HTMLElement {
    constructor() {
      super()
      this.tick = 0
      this.interval = null
    }
    connectedCallback() {
      this.render()
      this.interval = setInterval(() => {
        this.tick++
        this.render()
      }, 1000)
    }
    disconnectedCallback() {
      clearInterval(this.interval)
    }
    render() {

      this.innerHTML = `<div>Seconds Elapsed: ${this.tick}</div>`
    }
  });
  // Array.from(document.querySelectorAll(definition.tagName)).forEach(internals.Component);

};